#include <EthernetENC.h>
#include <EthernetUdp.h>

// MAC address (use any unique value)
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

// UDP settings
unsigned int localPort = 8888; // UDP listening port
EthernetUDP Udp;

// Packet buffer size
const int packetBufferSize = 255;
char packetBuffer[packetBufferSize];

// Solenoid pins and states
const int solenoidPins[] = {7, 6, 5, 4}; // Pins for solenoids
const int solenoidNotes[] = {36, 38, 40, 42}; // MIDI notes for solenoids
const int numSolenoids = sizeof(solenoidPins) / sizeof(solenoidPins[0]);
bool solenoidStates[numSolenoids] = {false, false, false, false};
unsigned long solenoidTimers[numSolenoids] = {0, 0, 0, 0};
const unsigned long solenoidDuration = 25; // Solenoid active time in ms

void cleanBuffer(char* buffer, int len) {
  int j = 0; // Index for cleaned buffer
  for (int i = 0; i < len; i++) {
    if (buffer[i] >= '0' && buffer[i] <= '9') {
      buffer[j++] = buffer[i]; // Keep numeric characters
    }
  }
  buffer[j] = '\0'; // Null-terminate the cleaned buffer
}

void setup() {
  Serial.begin(9600);
  Serial.println("Initializing Ethernet...");

  // Start Ethernet with DHCP
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP. Check network connection.");
    while (true); // Halt execution
  }

  // Print assigned IP address
  Serial.print("IP Address: ");
  Serial.println(Ethernet.localIP());

  // Start UDP
  Udp.begin(localPort);
  Serial.print("Listening on UDP port: ");
  Serial.println(localPort);

  // Set solenoid pins as outputs
  for (int i = 0; i < numSolenoids; i++) {
    pinMode(solenoidPins[i], OUTPUT);
    digitalWrite(solenoidPins[i], LOW); // Ensure solenoids are off
  }
}

void triggerSolenoid(int index) {
  solenoidStates[index] = true; // Set state to active
  solenoidTimers[index] = millis(); // Record activation time
  digitalWrite(solenoidPins[index], HIGH); // Activate solenoid
}

void checkSolenoids() {
  unsigned long currentMillis = millis();

  // Iterate through solenoids and deactivate if time is up
  for (int i = 0; i < numSolenoids; i++) {
    if (solenoidStates[i] && (currentMillis - solenoidTimers[i] >= solenoidDuration)) {
      solenoidStates[i] = false; // Reset state
      digitalWrite(solenoidPins[i], LOW); // Deactivate solenoid
    }
  }
}

void loop() {
  // Check for incoming UDP packets
  int packetSize = Udp.parsePacket();
  if (packetSize) {
    Serial.print("Received packet of size ");
    Serial.println(packetSize);

    // Read packet into buffer
    int len = Udp.read(packetBuffer, packetBufferSize - 1);
    if (len > 0) {
      packetBuffer[len] = '\0'; // Null-terminate the string

      // Debug: Print the buffer before cleaning
      Serial.print("Buffer before cleaning: ");
      Serial.println(packetBuffer);

      cleanBuffer(packetBuffer, len); // Remove non-numeric characters

      // Convert the cleaned buffer to an integer
      int receivedNumber = atoi(packetBuffer);

      // Debug: Print received number
      Serial.print("Received number: ");
      Serial.println(receivedNumber);

      // Check if the number matches a solenoid note
      for (int i = 0; i < numSolenoids; i++) {
        if (receivedNumber == solenoidNotes[i]) {
          Serial.print("Triggering solenoid on pin ");
          Serial.println(solenoidPins[i]);
          triggerSolenoid(i);
        }
      }
    }
  }

  // Check solenoid timers
  checkSolenoids();
}
